;=================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;=================================

ORG 4672

@Slow=0F2B			;ROM: SLOW subroutine
@DFile=400C			;16396
@LoadMenu=40E6			;16614
@SaveMenu=410E			;16654
@Wait=414A			;16714
@Release=4180			;16768
@KeyScan=418A			;16778
@Size=8000			;32768, Byte count of slideshow
@Table=8002			;32770, Table address, list of frame addresses
@Count=8004			;32772, Number of frames in slideshow
@Delay=8005			;32773, Initial framerate
@Frame01=800A			;32778, Address opf first frame

;=================================
;Initialize then start "forward" 
;slideshow.
;=================================
	CALL SaveMenu		;Save Listbox screen
	LD DE,Frame01		;Address of 1st frame
	CALL Decoder		;Display 1st frame
	CALL Slow		;Use SLOW mode
	LD A,(Delay)		;Get initial frame rate
	LD (Decoder+2),A	;Set initial frame rate
	JP Forward		;Start forward slideshow
;=================================
;Decode compressed data and display
;Modified routine by XavSnap
;On entry...
; DE=Frame to decode address
;=================================
@Decoder
	LD BC,0001		;Frame rate delay loop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ Decoder+3
	LD HL,(DFile)		;Get DFILE
	INC HL
@Expand
	LD A,01
	LD (SetVal+2),A		;Reset repeat counter to 1
	LD A,(DE)		;Get frame data
				;Invert chr. switch 
	LD B,A			;B=chr code
@Invert
	LD A,00			;0 or 128
	ADD A,B			;Add 0 or 128 to chr code
	BIT 6,A			;Multi-char?
	RES 6,A			;Remove multi-char tag
	LD (SetVal+1),A		;Set reg C to chr code
	JR Z SetVal		;Multi-char?
	INC DE			;Yes, set B counter
	LD A,(DE)		;Get multi-char repeat value
	LD (SetVal+2),A		;Set reg B to repeat value
	CP 01			;Test for end of frame sequence 40,01
	RET Z			;Return if end of frame
@SetVal
	LD BC,0100		;B=chr count, C=chr code
	LD (HL),C		;Print C
	INC HL			;Next DFile address
	BIT 6,(HL)		;N/L?
	JR Z +1			;No, repeat
	INC HL			;Yes, skip N/L
	DJNZ SetVal+3		;Loop until B=0
	INC DE			;Next frame addr
	JR Expand
;=================================
;Sort out the keyboard options
;=================================
@GetKeys
	CALL KeyScan
	CP 01			;No key being pressed?
	RET Z			;Yes, don't bother with the rest
@KeyN				;< Slower
	CP "N"
	JR NZ KeyM
	LD HL,Decoder+2		;Delay counter Hi byte
	INC (HL)		;Increase counter
	RET
@KeyM				;> Faster
	CP "M"			
	JR NZ KeyR
	LD HL,Decoder+2		;Delay counter Hi byte
	DEC (HL)
	RET P			;If negative then set back to 0
	INC (HL)		;Set back to 0 or delay will be FF
	RET
@KeyR				;Reverse direction
	CP "R"
	JR NZ KeyC
	CALL Release		;Wait for key release
@ForRev
	LD A,00			;Forward/Reverse switch
	XOR 01
	LD (ForRev+1),A
	JR NZ +5		;IF Z switch to HI-LO cycle
	LD HL,Forward		;LO-HI cycle
	JR SetRet
	LD HL,Reverse		;HI-LO cycle
@SetRet
	POP BC			;Clear stack
	POP BC
	POP BC
	PUSH HL			;Stack routine address
	RET			;RET to routine at HL
@KeyC				;Cycle
	CP "C"
	JR NZ KeyP
	LD HL,Cycle
	JR SetRet
@KeyP
	CP "P"			;PSI Res cycle
	JR NZ KeyO
@ISet
	LD A,FE			;Initialize A to FE
	INC A			;Set for next Ireg value
	INC A
	CP 20			;Out of ROM space?
	JR NZ +1		;No, move on
	XOR A			;Yes, cycle back to 0
	LD (ISet+1),A		;Store Ireg value
	LD I,A			;Set Ireg
	JP Release		;Return via wait for key release
@KeyO
	CP "O"			;Ireg switch
	JR NZ KeyS
	LD A,I			;Get current Ireg value
	CP 1E			;ROM Ireg?
	JR Z +4			;No, move on
	LD A,1E			;Restore Ireg to ROM value
	JR KeyO-5		;Set Ireg
	LD A,(ISet+1)		;Get last used Ireg value
	CP FE			;FE? Ireg not changed yet
	JR Z ISet+2		;Yes, set Ireg to 0
	JR KeyO-5		;No, set Ireg to last value
@KeyS				;Single step
	CP "S"
	JR NZ KeyI
	CALL Release		;Wait for key release
	LD HL,Step
	JR SetRet
@KeyI				;Invert
	CP "I"
	JR NZ KeyNL
	LD A,(Invert+1)		;Get invert switch, 0 or 128
	ADD A,80		;0+128=128 128+128=0
	LD (Invert+1),A		;Set invert switch
	JP Release		;Return via wait for key release
@KeyNL
	CP 76			;"N/L" key?
	JR Z +3			;Yes, return to basic
@KeyQ				;Quit to BASIC?
	CP "Q"		
	RET NZ
        LD A,1E			;Reset Ireg just in case
        LD I,A
	CALL LoadMenu		;Restore Listbox screen
	CALL Release		;Wait for key release
	RST 08
	FF
;=================================
;Forward cycle
;=================================
@Forward
	LD DE,Frame01		;Address of 1st frame
	LD A,(Count)		;Number of Slides to display
	LD B,A			;B=Frame counter
	PUSH BC			;Save frame counter
	CALL Decoder		;Display frame
	PUSH DE			;Save address pointer
	CALL GetKeys		;Check for Keypress
	POP DE			;Restore address pointer
	INC DE			;Start of next frame
	POP BC			;Restore frame counter
	DJNZ Forward+7		;Repeat till B=0
	JR Forward		;Start next loop
;=================================
;Reverse cycle
;=================================
@Reverse
	LD BC,(Count)		;Number of Slides to display
	LD B,00			;B=Frame counter
	LD HL,(Table)		;Slides Table
	ADD HL,BC
	ADD HL,BC 
	LD B,C			;Frame counter
@RevLoop
	PUSH BC			;PUSH frame counter
	DEC HL			;Move up the table
	LD D,(HL)		;DE=Address of frame to print
	DEC HL
	LD E,(HL)
	PUSH HL			;Push table pointer
	CALL Decoder		;Decode & Display frame
	CALL GetKeys		;Check for Keypress
	POP HL			;POP table pointer
	POP BC			;POP frame counter
	DJNZ RevLoop		;Repeat till B=0
	JR Reverse		;Restart loop
;=================================
;Forward then Reverse cycle
;=================================
@Cycle
	LD DE,Frame01		;Address of frame01
	LD A,(Count)		;Number of Slides to display
	LD B,A			;B=Frame counter
				;Start forward cycle
	PUSH BC			;Save frame counter
	CALL Decoder		;Decode & Display frame
	PUSH DE			;Save address pointer
	CALL GetKeys		;Check for Keypress
	POP DE			;Restore address pointer
	INC DE			;Start of next frame
	POP BC			;Restore frame counter
	DJNZ Cycle+7		;Repeat till B=0
				;Start of reverse cycle
	LD BC,(Count)		;Number of Slides to display
	LD B,00
        DEC C
	DEC C
	LD HL,(Table)		;Get start of Lookup Table
	ADD HL,BC		;Locate 2nd to last frames, skip last frame
	ADD HL,BC 			
	LD B,C			;Frame counter
@Recycle
	PUSH BC			;PUSH frame counter
	DEC HL
	LD D,(HL)		;DE=Address of frame to print
	DEC HL
	LD E,(HL)
	PUSH HL			;PUSH Table pointer
	CALL Decoder		;Decode & Display frame
	CALL GetKeys		;Check for Keypress
	POP HL			;POP table pointer
	POP BC			;POP frame counter
	DJNZ Recycle		;Loop until B=0
	LD HL,(Table)		;Get address of frame01
	INC HL			;Skip frame01
	INC HL
	LD E,(HL)		;Get frame02 LO Addr
	INC HL
	LD D,(HL)		;Get frame02 HI Addr
	LD A,(Count)		;Get number of slides to display
	DEC A			;Count-1
	JR Cycle+6
;=================================
;Single step
;=================================
@Step
	LD DE,Frame01		;Address of 1st frame
	LD A,(Count)		;Number of Slides to display
	LD B,A			;B=Frame counter
	PUSH BC			;Save frame counter
	CALL Decoder+8		;Decode & Display frame
	PUSH DE			;Save address pointer
	CALL Release		;Wait for key release
	CALL KeyScan		;Wait for keypress
	CP 01
	JR Z -7
	CP "S"			;S Pressed?
	JR NZ +11		;No
	CALL Release		;Wait for key release
	LD A,01			;Restart if "Forward" cycle
	LD (ForRev+1),A		;Reset ForRev flag
	CALL KeyR+4		;Yes, start Forward cycle
	POP DE			;Restore address pointer
	INC DE			;Start of next frame
	POP BC			;Restore frame counter
	DJNZ Step+7		;Repeat till B=0
	JR Step			;Repeat
