;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 4084

@PushReg=0220			;ROM: Push reg then start Display-1
@Disp3=0292			;ROM: DISPLAY BLANK LINES routine 
@DispRout=0281			;ROM: VIDEO-1 routine
@PopReg=02A4			;ROM: Pop reg then return control to program
@Keyboard=02BB			;ROM: KEYBOARD SCANNING subroutine
@Decode=07BD			;ROM: KEYBOARD DECODE subroutine
@Laddr=09D8			;ROM: LINE ADDRESS subroutine
@Reclaim1=0A5D			;ROM: RECLAIM routine, copy BC bytes starting at HL to DE
@Fast=0F23			;ROM: FAST subroutine
@Slow=0F2B			;ROM: SLOW subroutine
@Break=0F46			;ROM: Break-1 routine
@DFile=400C			;SYS VAR: Address of display file
@Pik=4021			;Pik memvar, listbox item number selection
@Seed=4032			;16 bit parameter passing via RAND XXXX
@Frames=4034			;SYS VAR: Frames counter
@HRDFile=8100			;Hires DFile
@HiRAM=8000			;Text to print
@WorkRAM=9900			;Scratchpad area

;================================
; WRX Video driver by Wilf Rigter
; WRX16K - 1996
; Annotations by Wilf Rigter
;================================
@WRX
	LD IX,HR		;Simple start of the hres mode
	RET
@LBuf
	LD R,A			;Load HFILE address LSB
	80 80 80 80		;32 NOPs = 256 pixels
	80 80 80 80 		;00=Black on white display
	80 80 80 80 		;80=White on black display
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	80 80 80 80 
	RET NZ			;Always returns
@HR    
	LD B,06			;Delay
@HR0
	DJNZ HR0
	NOP			;More delay fine tuning
	NOP
	NOP
	NOP
	NOP
	DEC B
@HDAddr
	LD HL,HRDFile		;HRDFILE
	LD DE,20		;32 bytes per line
	LD B,C0			;192 lines per hires screen
@HR1
	LD A,H			;Get HFILE address MSB
	LD I,A			;Load MSB into I register
	LD A,L			;Get HFILE address LSB
	CALL C089		;CALL Lbuf+8000
	ADD HL,DE		;Next line
	DEC B			;Dec line counter
	JP NZ HR1		;Last line
@HR2   
	CALL Disp3		;Return to application program
	CALL PushReg		;Extra register PUSH and VSYNC
               			;This code segment is optional
@Brake
	CALL Break		;Check break key
	LD A,1E			;Restore pattern table pointer
	LD I,A
	JR NC Stop		;Skip the HR vector load if BREAK
	LD IX,HR		;Load the HR vector
@Stop
	JP PopReg		;Return to application program
@Format
	00			;Print format
	00
	00
	00
	00
	00
;================================
@JumpTable
	JR LoadMenu		;16614
	JR Sortkeys		;16616
	JP ListUp		;16618
	JP ListDown		;16621
	JP Info			;16624
	JP ASCII		;16627
	JP HRPrint		;16630
	JP ZXScreen		;16633
	JP BMP256x192		;16636
	JP SCRtoZX		;16639
	JP Delete		;16642
	00 00 00 		;16645
	JP BMP256x384		;16648
	JP FontView		;16651
	JP SaveMenu		;16654
;================================
; Restore lores display
;================================
@Lores
	HALT			;Wait for an interrupt - used to sync the changeover!
	LD A,(Frames) 		;Wait for the next new tv frame
	LD C,A
@Sync
	LD A,(Frames)
	CP C
	JR Z Sync
	LD A,1E		 	;Reset the "I" Register
	LD I,A		 	;to the ROM default (30 decimal)
	LD IX,DispRout	 	;Pointer to the ROM display routine set!
	JR Ret0
;================================
; Save menu screen
;================================
@SaveMenu
	LD HL,0002		;Locate screen storage area
	CALL LAddr		;Use ROM routine to find line address
	LD BC,000C		;Skip header
	ADD HL,BC
	EX DE,HL		;DE=Destination
	LD HL,(DFile)		;HL=Source
@Save01
	LD BC,0319		;BC=byte count
	LDIR			;Copy source to desination
	RET			;RET to BASIC, BC=0000
;================================
; Load menu screen
;================================
@LoadMenu
	LD HL,0002		;Locate screen storage area
	CALL LAddr		;Use ROM routine to find line address
	LD BC,000C		;Skip header
	ADD HL,BC		;HL=screen store=source
	LD DE,(DFile)		;DE=DFile=Destination
	JR Save01
;================================
; Wait for key press
; On exit...
;  A=key chr code
;================================
@Wait
	CALL KeyScan		;Get keypress
	CP 01			;No key?
	JR Z Wait		;Yes, loop until a keypressed
	RET
;================================
; Keyboard loop
;================================
@SortKeys
	CALL Wait
	LD BC,0001		;C=1
	CP "M"			;Key "M"? Pointer down
	RET Z			;Yes, return 1
	CP "6"			;Key "6"? Pointer down
	RET Z			;Yes, return 1
        INC C			;C=2
	CP "K"			;Key "K"? Pointer up
	RET Z			;Yes, return 2
	CP "7"			;Key "7"? Pointer up
	RET Z			;Yes, return 2
        INC C			;C=3
	CP "D"			;Key "D"? Page down
	RET Z			;Yes, return 3
	CP "8"			;Key "8"? Page down
	RET Z			;Yes, return 3
        INC C			;C=4
	CP "U"			;Key "U"? Page up
	RET Z			;Yes, return 4
	CP "5"			;Key "5"? Page up
	RET Z			;Yes, return 4
        INC C			;C=5
	CP 76			;Key N/L? Select Item
	RET Z			;Yes, return 5
        INC C			;C=6
	CP "Q"			;Key "Q", Quit
	RET Z			;Yes, return 6
	CP "0"			;Key "0", Quit
	RET Z			;Yes, return 6
	JR Sortkeys		;Loop until valid key pressed
;================================
; Wait for key release
;================================
@Release
	CALL Keyboard		;Call ROM scan keyboard routine.
	INC L			;Check if user's finger on key.
	JR NZ Release		;If so, go back and check again.
@Ret0
	LD BC,0000
	RET			;RET to BASIC, BC=0000
;================================
; Test for keypress.
; On exit...
;  A=key chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; Scroll listbox up
;================================
@ListUp
	LD HL,(DFile)		;Get DFile
	LD DE,0066		;Offset to NW corner, 102
	ADD HL,DE		;NW corner
	LD B,12			;Rows to scroll
	LD A,B			;Save row counter in A
	LD BC,0021		;Bytes/ROW, 33
	LD D,H			;DE=destination
	LD E,L
	ADD HL,BC		;HL=source
	PUSH HL			;PUSH source
	LD C,0C			;Characters/row, 12
 	LDIR			;Move row up
	POP HL			;POP source
	LD B,A			;B=row counter
	DJNZ Listup+9		;Loop until B=0
	RET			;RET to BASIC, BC=0000
;================================
; Scroll listbox down
;================================
@ListDown
	LD HL,(DFile)		;Get DFile
	LD DE,02C3		;Offset to SW corner, 707
	ADD HL,DE		;SW corner
	LD B,12			;ROW count
	LD A,B			;Save row counter in A
	LD BC,0021		;Bytes/ROW, 33
	LD D,H			;DE=destination
	LD E,L		
	AND A			;Clear carry
	SBC HL,BC		;HL=source addr.
	PUSH HL			;PUSH source
	LD C,0C			;Characters/row
 	LDDR			;Move row up
	POP HL			;POP source
	LD B,A			;B=row counter
	DJNZ ListDown+9		;Loop until B=0
	RET			;RET to BASIC, BC=0000
;================================
; Clear the HRDFile
;================================
@HRCLS
	LD HL,HRDFILE		;Source=HRDFile address
	LD (HL),00		;Set to 0
	LD D,H
	LD E,L
	INC DE			;Destination=HL+1
	LD BC,17FF		;Byte count=HRDFile size-1
	LDIR			;Clear the HRDfile
	RET			;RET to BASIC, BC=0000
;================================
; WRX invert screen
;================================
@Invert
	PUSH HL			;Save main reg, EXX doesn't seem to work for some reason
	PUSH DE
	PUSH BC
	LD HL,Lbuf+2		;Source=WRX hires line buffer+2
	LD BC,001F		;Byte count, 31
	LD A,(HL) 		;Invert the value
	ADD A,80 		;by adding 128
	LD (HL),A		;Load the inverted value 
	LD D,H 
	LD E,L 
	INC DE 			;Set destination
	LDIR 
	POP BC			;Restore main reg
	POP DE
	POP HL
	RET
;================================
; Print Info
; On entry...
;  (Seed)=XXXX REM line number
;  where the the info is stored
;  for the PIKed selection.
;================================
@Info
	CALL EraseText		;Erase textbox
	LD HL,03E8		;HL=1000
	LD A,(Pik)		;Line number=1000+A
	LD B,00			;BC line number offset, 1 to 255
	LD C,A
	ADD HL,BC		;HL=Info Line number
	CALL LAddr		;Use ROM routine to find line address
	JP NZ Ret0		;Return because line not found
	LD BC,0005		;Skip header
	ADD HL,BC		;HL=Start address of note
	EX DE,HL		;DE=Start of title section
	LD A,(DE)		;Get chr to print
	CALL EOLTest
	JR Z Comment		;Yes, no title section, start comment section
	LD BC,1702		;Y/X coord, 23/02, of title line
	CALL ZXPrint		;Print chr
@Info01
	INC HL			;Next title line print address
	INC DE			;Next chr to print
	CALL EOLTest		;Test for EOL/EOF chr
	JR Z Comment		;IF Z THEN EOL chr, start printing comment section
	BIT 2,(HL)		;Right margin chr?
	JR Z +1			;No, print chr 
	DEC HL			;Yes, backup print position
	LD (HL),A		;Print chr
	JR Info01		;Loop until EOL/EOF reached
@Comment
	INC DE			;Start of comment section
	LD BC,030F		;Y/X coords, 03/15, of NW corner of textbox
@Info02
	XOR A			;Locate print position
	CALL ZXPrint		;Print chr
	CALL EOLTest		;Test for EOL/EOF chr
	INC DE			;Next data byte
	JR Z Info03		;Yes, start next row
	LD (HL),A		;Print info text
	INC HL			;Next DFile address
	LD A,(HL)		;Test for right margin of textbox
	CP 05			;Right margin?
	JR NZ Info02+4		;No, loop until margin or EOL marker found
@Info03
	INC B			;Start next row
	LD A,16			;Last row?
	CP B
	JR NZ Info02		;No, Loop until EOF or last row
	JR ZXPrint-4
;================================
; Check for EOL marker
; On entry...
;  DE=Chr to print address
;  HL=DFile address
; On exit...
;  A=Chr to print
;  F=Z if not EOL
;  F=NZ if not EOL
;================================
@EolTest
	LD A,(DE)		;Get chr to print
	CP 8C			;EOL marker, "g"?
	RET Z			;Ret if EOL marker
	CP 76			;EOF marker?
	RET NZ			;No, keep printing
	POP HL			;Yes, Clear stack
	LD BC,0000
	RET			;Ret 0 to BASIC
;================================
; Locate print position in DFile
; then print Reg A chr.
; On entry...
;  A=chr. to print
;  B=Ycoord
;  C=Xcoord
; On exit
;  HL=DFile address
;  DE, BC, AF unchanged
;================================
@ZXPrint
	PUSH DE			;PUSH print position
	PUSH BC			;PUSH Y/Xcoords
	INC B			;Ycoord must be > 0
	LD HL,(DFile)		;Get start of DFile
	INC HL
	LD DE,0000		;Initialize DE
	ADD HL,DE		
	LD E,21			;DE=33 bytes/row
	DJNZ -5			;Add Bx33 to start of DFile
	LD E,C			;DE=Xcoord offset
	ADD HL,DE		;Add offset to HL
@PokeA
	LD (HL),A		;Print A
	POP BC			;POP Y/Xcoords
	POP DE			;POP print position
	RET
;================================
; Erase the text box area and
; title line
;================================
@EraseText
	LD BC,030F		;Y/X coord, 03/15, NW corner of text box
	XOR A			;A=0
	CALL ZXPrint		;Print " " & return DFile address 
	INC HL			;Next column to erase
	LD A,(HL)		;Test for right margin of text box
	CP 05			;Right margin?
	JR Z +4			;Yes, move on
	LD (HL),00		;Print " "
	JR EraseText+7		;Loop until right margin reached
	INC B			;Increase Y coord, next row
	LD A,B			;Test row number
	CP 16			;Done?
	JR NZ EraseText+3	;Loop until B=16
	LD A,80			;Clear title line
	LD BC,1702		;Y/X coord, 23/02, of title line
	CALL ZXPrint		;Print "g " & return DFile address 
	LD B,1C			;Byte counter, 28
	INC HL			;Next location
	LD (HL),A		;Clear location
	DJNZ -4			;Loop until B=0
	RET
;================================
; 256x192 BMP routine
;================================
@BMP256x192
	LD HL,VFlip+8		;Install vert. flip parameters for 256x192 BMP
	LD (HL),17
	LD HL,WF01-1
	LD (HL),60
	CALL VFlip		;Flip the image
@SCRDisplay
	CALL Slow
	CALL WRX		;Start hires display
@K192
	CALL Release		;Wait for no keypress
	CALL Wait		;Wait for keypress
	CP "Q"			;Q Key?
	JR Z +4			;Yes, return
	CP 76			;N/L key?
	JR NZ +6		;No, move on
	CALL Release		;Wait for key release
	JP Lores		;Yes, start lores mode and return to BASIC
	CP "I"			;I Key?
	JR NZ +5		;No, move on
	CALL Invert		;Yes, invert screen
	JR K384
	CP "V"			;V Key?
	JR NZ +3		;No, move on
	CALL VFlip		;Vertical flip
	JR K192			;Loop until N/L pressed
;================================
; 256x384 BMP routine
;================================
@BMP256x384
	LD HL,VFlip+8		;Install vert. flip parameters for 256x384 BMP
	LD (HL),2F
	LD HL,WF01-1
	LD (HL),C0		;Row counter
	CALL VFlip		;Flip the image
;	CALL Invert		;Swap B<->W
	CALL Slow
	CALL WRX		;Start hires display
	CALL PanUp
	CALL PanDown
@K384
	CALL Release		;Wait for no keypress
	CALL Wait		;Wait for keypress
	CP "Q"			;Q Key?
	JR Z +4			;Yes, return
	CP 76			;N/L key?
	JR NZ +11		;No, move on
	CALL Release		;Wait for key release
	LD A,81
	LD (HDAddr+2),A		;Reset HRDFile pointer
	JP Lores		;Yes, start lores mode and return to BASIC
	CP "I"			;I Key?
	JR NZ +5		;No, move on
	CALL Invert		;Yes, invert screen
	JR K384
	CP "V"			;V Key?
	JR NZ +5		;No, move on
	CALL VFlip		;Vertical flip
	JR K384
	CP "N"			;Pan down
	JR NZ +5
	CALL PanDown
	JR K384
	CP "M"			;Pan up
	JR NZ +3
	CALL PanUp
	JR K384			;Loop until "Q" or N/L pressed
;================================
; WRX vertical flip
;================================
@VFlip
	LD HL,HRDFile		;Start of HRDFile
	LD (HiRAM),HL		;Save top row addr
	LD BC,17E0		;BC=191 rows * 32 Bytes/row
	ADD HL,BC		;HL=Address of bottom row.
	LD B,60			;Rows to process counter
@WF01
	PUSH BC			;PUSH counter
	LD (HiRAM+2),HL		;Save bottom row addr
	LD HL,(HiRAM)		;Transfer
	LD DE,HiRAM+4		;top row data
	LD BC,0020		;to
	LDIR			;storage area
	LD HL,(HiRAM)		;Transfer
	EX DE,HL		;bottom row
	LD HL,(HiRAM+2)		;data
	LD C,20			;to
	LDIR			;top row address
	LD (HiRAM),DE		;Save top row address
	LD HL,HiRAM+4		;Transfer
	LD DE,(HiRAM+2)		;top row data
	LD C,20			;to
	LDIR			;bottom row address
	LD C,20			;Calculate 
	LD HL,(HiRAM+2)		;address
	XOR A			;of
	SBC HL,BC		;next bottom row
	POP BC			;POP row counter
	DJNZ WF01		;Repeat if NZ
	RET
;================================
;Pan Up
;================================
@PanUp
	LD A,(HDAddr+2)
	CP 99
	RET Z
	LD HL,HRDFile
	LD B,C0			;Lines to move
	LD DE,0020
	ADD HL,DE
	LD (HDAddr+1),HL	;Change HRDFile pointer
	CALL PAUSE
	DJNZ -9
	RET
;================================
;Pan Down
;================================
@PanDown
	LD A,(HDAddr+2)
	CP 81
	RET Z
	LD HL,(HDAddr+1)
	LD B,C0			;Lines to move
	LD DE,0020
	AND A			;Clear carry
	SBC HL,DE
	LD (HDAddr+1),HL	;Change HRDFile pointer
	CALL PAUSE
	DJNZ -11
	RET
;================================
;Pause the pan 
;================================
@Pause
	PUSH HL			;Save main reg, EXX doesn't seem to work for some reason
	PUSH DE
	PUSH BC
	LD BC,0064 		;Delay Loop
	CALL Delay
	CALL Release
	POP BC
	POP DE
	POP HL
	RET
;================================
@Delay
	DEC BC 
	LD A,B 
	OR C 
	JR NZ Delay
	RET
;===============================
; Spectrum SCR to ZX81 conversion
;===============================
@SCRtoZX
	LD B,03			;Counter, 3 thirds to process
	LD HL,WorkRAM		;Source, SCR dump data
	LD DE,HRDFile		;Destination
	PUSH BC			;PUSH thirds counter
	PUSH HL			;PUSH source address
	LD A,08			;Counter, 8 chr rows/SCR third
@SCRLoop1
	PUSH AF			;PUSH Chr rows counter
	LD A,08			;Counter, 8 pixel rows/chr row
	PUSH HL			;PUSH SCR dump address
@SCRLoop2
	LD (CopySCR+1),HL	;Set source
@CopySCR
	LD HL,0000		;Source, SCR dump data
	LD BC,0020		;Byte count, 32 bytes/row
	LDIR			;Copy row to HRDFile
	LD BC,00E0		;Offset to next pixel row, 224
	ADD HL,BC		;Next pixel row source
	DEC A			;Decrease pixel row counter
	JR NZ SCRLoop2		;Do 8 pixel rows
	POP HL			;POP SCR dump address
	LD C,20			;32 bytes/row
	ADD HL,BC		;Next pixel row in chr row
	POP AF			;Chr row counter
	DEC A			;Decrease chr row counter
	JR NZ SCRLoop1		;Do 8 chr rows
	POP HL			;POP source address
	LD BC,0800		;Offset to next third, 2048 bytes
	ADD HL,BC		;Set Source address
	POP BC			;POP thirds done counter
	DJNZ SCRtoZX+8		;Loop until B=0
	JP SCRDisplay		;Display image
;================================
; ZX display Screen
;================================
@ZXScreen
	CALL Savemenu		;Save menu screen
	CALL Slow
	LD HL,WorkRAM		;HL=Source=ZX screen
	LD DE,(DFile)		;DE=DFile=Destination
	INC DE
	LD BC,0318
	LDIR
@ZXKeys
	CALL Release		;Wait for no keypress
	CALL Wait		;Check for keypress
	CP "Q"			;"Q" key?
	JR Z ZXQuit
	CP 76			;N/L Key?
	JR Z ZXQuit
	CALL ZXInvert		;Test for "I" key
	JR ZXKeys		;Loop until "Q" or N/L pressed
@ZXQuit
	CALL Release
	JP LoadMenu
;================================
; ZX invert Screen
;================================
@ZXInvert
	CP "I"			;"I" key?
	RET NZ			;No, return
	LD BC,0300 		;Byte count
	LD HL,(DFile)		;Get start of DFile
@ZXInv01
	INC HL
	LD A,(HL)		;Load chr. code
	CP 76			;N/L chr?
	JR Z ZXInv01		;Yes, skip it by looping back
	ADD A,80		;Add 128 to chr. code
	LD (HL),A		;Display inverted chr. code
	DEC BC 			;Decrease byte counter
	LD A,B 
	OR C 
	JR NZ ZXInv01		;Loop until BC=0
	RET			;Ret 0 to BASIC
;================================
; Convert ASCII to ZX81 format
;================================
@ASCII
	LD HL,WorkRAM-1		;Start of ASCII data-1
@ASC01
	INC HL
	LD A,(HL)
	CP 2A			;ASCII *?
	JR NZ LineFeed		;No, test for linefeed chr
	INC HL			;Yes, test for *E or *e?
	LD A,(HL)		;
	CP 45			;ASCII E?
	JR Z +2
	CP 65			;ASCII e?
	DEC HL
	LD A,2A			;Print * chr
	JR NZ LineFeed		;No, not EOF
	LD (HL),FF		;New EOF Marker
	INC HL			;Skip ASCII E
	INC HL			;Skip ASCII O
	INC HL			;Skip ASCII F
	INC HL			;Get format chr
	LD A,(HL)		;3=33, 6=36
	LD (Format),A
	LD BC,0000
	RET			;Ret 0 to BASIC
@LineFeed
	CP 0D			;Line feed
	JR NZ NotAlpha
	LD (HL),76		;Replace ASCII EOL chr with ZX EOL chr
	INC HL
	LD (HL),00		;Replace ASCII RETURN chr with space
	JR ASC01
@NotAlpha			
	CP 41			;ASCII code<A?
	JR C Numeral
@Upper				;A to Z
	CP 41			;ASCII code<A
	JR C Lower
	CP 5B
	JR NC Lower		;ASCII code>Z
	ADD A,65
	JR Change
@Lower				;a to z
	CP 61			;ASCII code<a
	JR C Symbol
	CP 7B			;ASCII code>z
	JR NC Symbol
	SUB 3B
	JR Change
@Numeral			;0 to 9
	CP 30			;ASCII code<0
	JR C Symbol
	CP 3A			;ASCII code>9
	JR NC Symbol
	SUB 14
	JR Change
@Symbol				;Symbols
	EX DE,HL		;Save char address
	LD BC,0046		;2*Characters to compare
	LD HL,Symbolz
	CPIR			;Locate A in table
	LD A,(HL)		;Get the conversion value
	EX DE,HL		;Restore chr address
	JR Z Change		;Unsupported ASCII chr
@NoGood
	LD A,8F			;Unsupported ASCII chr. print "g?"
@Change
	LD (HL),A	
	JR ASC01
;================================
@Symbolz
	20 00			; Space
	22 0B			; "
	A3 0C			; 
	24 0D			; $
	3A 0E			; :
	3F 0F			; ?
	28 10			; (
	29 11			; )
	3E 12			; >
	3C 13			; <
	3D 14			; =
	2B 15			; +
	2D 16			; -
	2A 17			; *
	2F 18			; /
	3B 19			; ;
	2C 1A			; ,
	2E 1B			; .
;================================
;Replacement symbols
	09 00			; TAB with space		
	7E 99			; ~ with g;
	60 8F			; ` with g?
	21 8B			; ! with g"
	40 95			; @ with g+
	23 8C			; # with g
	25 8D			; % with g$
	5E 93			; ^ with g<
	26 8E			; & with g:
	5F 94			; _ with g=
	5B 90			; [ with g(
	7B 96			; { with g-
	5D 92			; ] with g>
	7D 98			; } with g/
	5C 91			; \ with g)
	7C 97			; | with g*
	27 8F			; ' with g?
;================================
; Locate chr pattern in font table
; On entry...
;  A=Chr code
; On exit
;  DE=Pattern location
;================================
@FindChr
	LD C,A			;BC=chr code 
@Font
	LD HL,0000		;POKEd FONT32 base address
	RL C 			;Code*2
	RL B
	RL C 			;Code*4
	RL B 
	RL C 			;Code*8
	RL B 
	ADD HL,BC 		;HL=Pattern pointer
	EX DE,HL 		;DE=Pattern pointer
	RET
;================================
; Print text file
;================================
@HRPrint
	CALL HRCLS		;Clear HRDFile
	CALL Slow
	CALL WRX		;Start WRX
	LD A,(Format)		;Get print format
	CP 33			;ASCII 3=32x24
	JR Z Print32		;Print 32 col text
	CP 36			;ASCII 6=64x24
	JR NZ 04		;No, move on
	LD A,01			;Formatted text
	JR 01
	XOR A			;Unformated text
	LD HL,NL64-1		;Set JR value
	LD (HL),A
@Print64
	LD HL,0003		;Locate line 3
	CALL LAddr		;Use ROM routine to find line address
	LD BC,040D		;Offset to Font64 base address
	ADD HL,BC		;HL=Font64 base address
	LD (Font+1),HL		;POKE font pointer
	LD DE,WorkRAM-1		;Text to print-1
	LD B,00			;Ycoord=0
@Line64
	LD C,00			;Xcoord=0
	LD A,C			;A=Xcoord
	LD (Xcoord+1),A		;POKE Xcoord
	INC DE			;Increase text pointer
	LD A,(DE)		;Get chr to print
	CP 76			;Newline?
	JR NZ 03		;No, move on
	INC DE			;Yes, increase text pointer, skips the N/L
	JR NL64			;Start next line
	CP FF			;EOF?
	JR Z EOFRet
	CALL PrintC64		;Print chr
	INC C			;Increase Xcoord
	BIT 6,C			;End of line?
	JR Z Line64+2 		;No, print next chr
	LD A,(DE)		;Text Newline?
	BIT 6,A			;Test chr
	JR Z 01			;No, move on
@NL64
	INC B			;Increase Ycoord
	LD A,18			;24 rows/screen
	CP B			;End of page?
	JR NZ Line64		;Loop until B=24
	CALL TxtKeys
	JR Line64		;Loop until EOF reached
;================================
; Test text keys
;================================
@TxtKeys
	PUSH DE			;PUSH text pointer
	CALL Wait		;Wait for keypress
	CP "I"			;I Key?
	JR NZ +3		;No, move on
	CALL Invert		;Yes, invert screen
	CP "Q"			;"Q" pressed?
	JR NZ NotQ		;No, return
	POP DE			;Clear stack
	POP DE			
	JR +3
@EOFRet
	CALL Wait		;Wait for keypress
	CALL Release		;Wait for key release
	JP Lores		;Start lores mode and return to basic
@NotQ
	CALL HRCLS		;Clear the HRDFile, BC=0000
	POP DE			;Clear stack
	RET
;================================
; Print 32 col text file to HRDFile
;================================
@Print32
	LD HL,0003		;Locate line 3
	CALL LAddr		;Use ROM routine to find line address
	LD BC,000D		;Offset to Font32 base address
	ADD HL,BC		;HL=Font32 base address
@Font32
	LD (Font+1),HL		;Set font pointer
	LD DE,WorkRAM-1		;Text to print-1
	LD B,00			;Ycoord=0
@Line32
	LD C,00			;Xcoord=0
	INC DE
	LD A,(DE)		;Chr to print
	CP 76			;Text Newline?
	JR NZ +3		;No, move on
	INC DE			;Yes, increase text pointer, skips the N/L
	JR NL32
	CP FF			;EOF?
	JR Z EOFRet
	CALL PrintC32		;Print chr code A
	INC C			;Increase Xcoord
	LD A,C
	CP 20			;End of line?
	JR NZ Line32+2 		;No, print next chr
	LD A,(DE)		;Text Newline?
	BIT 6,A			;
	JR Z +1			;No, move on
@NL32
	INC B			;Yes, Increase Ycoord
	LD A,18			;24 rows/screen
	CP B			;End of page?
	JR NZ Line32		;Loop until B=24
	CALL TxtKeys
	JR Line32		;Loop until EOF reached
;================================
; Preview font
;================================
@FontView
	CALL HRCLS		;Clear HRDFile
	CALL Slow
	CALL WRX		;Start WRX
	LD HL,9D00		;Font Location
	JR Font32
;================================
; Locate print position in HRDFile
; then print chr. A
; On entry...
;  A=chr. to print
;  B=Ycoord
;  C=Xcoord
; On exit
;  AF, BC, DE unchanged
;================================
@PrintC32
	PUSH DE			;PUSH text pointer
	PUSH BC			;PUSH X/Ycoords
	INC B			;Ycoord must be > 0
	LD HL,HRDFile		;Get start of DFile
	DEC H
	LD DE,0100		;DE=256 bytes/row
	ADD HL,DE		
	DJNZ -3			;Add Bx256 to start of HRDFile
	LD D,E			;Add Xcoord offset to HL
	LD E,C			;DE=Xcoord offset
	ADD HL,DE		;Add offset to HL
	PUSH HL 		;PUSH HRDFile addr
@Chr32
	BIT 7,A			;Uppercase chr?
	JR Z +2			;No, move on
	SUB 40	 		;Subtract 64 from character code
	CALL FindChr		;Locate chr pattern
	POP HL 			;POP HRDFile addr
	LD B,08 		;Set counter
@Row32
	PUSH BC 		;PUSH counter
	LD A,(DE) 		;Get Font pattern
	LD (HL),A 		;Print results
	INC DE 			;Increase font pointer
	LD BC,0020 		;32 bytes/row
	ADD HL,BC 		;Next HRDFile pixel row
	POP BC 			;POP counter
	DJNZ Row32		;Loop until B=0
	POP BC			;POP X/Ycoords
	POP DE			;POP text pointer
	RET
;================================
; Locate print position in HRDFile
; then print Reg A chr.
; On entry...
;  A=chr. to print
;  B=Ycoord
;  C=Xcoord
; On exit...
;  BC, DE unchanged
;================================
@PrintC64
	PUSH BC			;PUSH X/Ycoords
	PUSH DE			;PUSH text pointer
	INC B			;Ycoord must be > 0
	LD HL,HRDFile		;Get start of DFile
	DEC H
	LD DE,0100		;DE=256 bytes/row
	ADD HL,DE		
	DJNZ -3			;Add Bx256 to start of HRDFile
	LD D,E			;Add Xcoord offset to HL
	SRL C			;Xcoord=Xcoord/2
	LD E,C			;DE=Xcoord offset
	ADD HL,DE		;Add offset to HL
	PUSH HL 		;PUSH HRDFile addr
@Chr64
	BIT 7,A			;Uppercase chr?
	PUSH AF			;PUSH Z flag
	JR Z +2			;No, move on
	SUB 80	 		;Subtract 128 from character code
	CALL FindChr		;Locate chr pattern
	POP AF			;POP Z flag
	POP HL 			;POP HRDFile addr
@Xcoord
	LD A,00			;POKEd Xcoord, 0 to 63
	RRCA			;C=0=Left chr   C=1=Rite chr			
	LD B,08 		;Set counter
@Row64
	PUSH BC 		;PUSH counter
	PUSH AF 		;PUSH left/rite flag
	LD A,(DE) 		;Get Font pattern
	LD B,(HL) 		;Get HRDFile pattern
	JR Z LeftLower		;Results of AND A Z=Lowercase NZ=Uppercase
	JR C ShiftRite		;
	AND F0 			;AND 11110000, A=Uppercase pattern
@LeftChr
	PUSH AF			;PUSH Font pattern 
	LD A,B 			;B=HRDFile pattern
	AND 0F 			;AND 00001111, A=Right Nybble of HRDFile pattern
@RiteChr
	LD B,A 			;B=Right Nybble of HRDFile pattern
	POP AF 			;POP Font pattern
	OR B 			;Merge font pattern with HRDFile pattern
	LD (HL),A 		;Print results
	INC DE 			;Increase font pointer
	LD BC,0020 		;32 bytes/row
	ADD HL,BC 		;Next HRDFile pixel row
	POP AF 			;POP left/rite flag
	POP BC 			;POP counter
	DJNZ Row64		;Loop until B=0
	POP DE			;POP text pointer
	POP BC			;POP X/Ycoords
	RET
@ShiftRite
	SRL A 			;Shift font pattern right
	SRL A 
	SRL A 
	SRL A 
	PUSH AF 		;PUSH shifted pattern
	LD A,B 			;A=HRDFile pattern
	AND F0 			;AND 11110000, A=Left font pattern
	JR RiteChr
@LeftLower
	JR C RiteLower
	SLA A 			;Shift font pattern left
	SLA A 
	SLA A 
	SLA A 
	JR LeftChr
@RiteLower
	AND 0F 
	JR ShiftRite+8
;================================
; Delete line numbers 9000 to 9980
; in steps of 10
;================================
@Delete
	CALL Fast		;Go to FAST mode
	LD HL,2328		;Start at line 9000
@GetLine
	PUSH HL			;Save line number
	CALL LAddr		;Use ROM routine to find line address
	JR Z LineFound
	POP HL			;POP line number
	LD BC,000A		;Line increments of 10
	ADD HL,BC
	LD A,27			;Done when HL=9990=$2706
	CP H
	JR NZ GetLine
	CALL Slow
	LD BC,0000
	RET
@LineFound
	PUSH HL			;PUSH line address
	INC HL 			;Skip line number lo
	INC HL 			;Line number hi addr.
	LD C,(HL) 		;Line size lo value
	INC HL 			;Line size hi value addr
	LD B,(HL) 		;BC=Byte count to reclaim
	ADD HL,BC 		;
	INC HL 			;HL=Source, end of line+1
	POP DE			;POP line address, DE=Destinatin
	CALL Reclaim1		;Reclaim routine, copy BC bytes starting at HL to DE
	JR GetLine+6
	NOP