;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;         Music  Animations
;      Gregory C. Harder, 2019
;================================
ORG 4D08

@Keyboard=02BB			;ROM: KEYBOARD SCANNING subroutine
@Decode=07BD			;ROM: KEYBOARD DECODE subroutine
@Laddr=09D8			;ROM: LINE ADDRESS subroutine
@Seed=4032			;16 bit parameter passing via RAND XXXX
@Frames=4034			;SYS VAR: Frame counter
@Pattern=403C			;16444, 
@Cell=403C			;16444, Cell character
@BKG=403D			;16445, background character
@Density=403E			;16446, Cell density
@DFile=400C			;SYS VAR: DFile address
@Workarea=49A1			;Workarea
@altDFile=49C2			;Copy of DFile

@JumpTable
	JR SymmA		;19720
	JR SymmB		;19722
	JP RandomScr		;19724
	JP Scope		;19727
	JP Wallpaper		;19730
	JR LifeA		;19733
	JR LifeB		;19735
	JR LifeC		;19737
	JP CLS			;19739
;================================
@GLoop
	CALL Global		;Run one Global gen
	CALL KeySort		;Check for keypress
	JR GLoop		;Next Gen
;================================
; Apply LIFE algorithm to SymmA
;================================
@LifeA
	CALL Release		;Wait for no key press
	CALL CLS
	CALL SymA
	LD HL,Density		;Get loop counter
	DEC (HL)		;Decrease counter
	JR NZ LifeA+6		;Loop until counter=0
	JR GLoop
;================================
; Apply LIFE algorithm to SymmB
;================================
@LifeB
	CALL Release		;Wait for no key press
	CALL CLS
	CALL SymB
	LD HL,Density		;Get loop counter
	DEC (HL)		;Decrease counter
	JR NZ LifeB+6		;Loop until counter=0
	JR Gloop
;================================
; Apply LIFE algorithm to random
; screen
;================================
@LifeC
	LD A,C9			;Return instruction
	LD (RandRet),A		;Set for return instead of loop
	CALL RandomScr+5
	JR GLoop
;================================
; Four fold symmetry, style A
;================================
@SymmA
	CALL Release		;Wait for no key press
	CALL SymA
	CALL KeySort		;Test keys
        JR SymmA+3
;================================
@SymA
	LD A,0C			;Upper limit=12
	LD (Limit+1),A		;POKE upper limit
	CALL Modulus
	LD B,A          	;B=Ycoord
	LD A,10			;Upper limit=16
	LD (Limit+1),A		;POKE upper limit
	CALL Modulus
        LD C,A          	;C=Xcoord
	CALL PrintCell		;Get chr to print, (BKG) or (Cell)
	JP PrintQuads		;Print A to the four quadrants
;================================
; Four fold symmetry, style B
;================================
@SymmB
	CALL Release		;Wait for no key press
	CALL SymB
	CALL KeySort		;Test keys
        JR SymmB+3
;================================
@SymB
	CALL PrintCell		;Get chr to print, (BKG) or (Cell)
	PUSH AF			;PUSH chr to print
	LD A,10			;Upper limit=16
	LD (Limit+1),A		;POKE upper limit
	CALL Modulus
	LD B,A			;Set Ycoord
	LD A,10			;Upper limit=16
	LD (Limit+1),A		;POKE upper limit
	CALL Modulus
	LD C,A			;Set Xcoord
	POP AF			;POP chr to print
	PUSH BC
	CALL PrintQuads
	POP BC			;POP Y/Xcoords
	LD A,B			;Swap Y/Xcoords
	LD B,C
	LD C,A
	JP PrintQuads
;================================
; Generate a 32x24 random screen
;================================
@RandomScr
	LD A,18			;JR instruction
	LD (RandRet),A		;Set for loop instead of return
	LD HL,altDFile+1	;Get start of altDFile
	LD BC,0300		;Byte count
@RSLoop
	PUSH HL			;PUSH altDFile address
	CALL Rnd8Bit		;Get random number
	LD L,A			;L=8-Bit random number, range 0 to 255
	LD A,(Density)		;Get density value
	CP L			;IF L>Density THEN PRINT Cell ELSE PRINT Bkg
	LD A,(Cell)		;A=Cell
	JR C +3			;If CY print cell
	LD A,(BKG)		;No, print bkg
	POP HL			;POP altDFile address
	LD (HL),A		;Print cell or bkg
	INC HL			;Next altDFile address
	BIT 6,(HL)		;N/L?
	JR Z +1			;No, move on
	INC HL			;Yes, Skip it
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ RSLoop		;Loop until BC=0
	CALL SwapDF		;Display results
	CALL KeySort		;Test keys
@RandRet
	JR RandomScr+5
;================================
; Kaleidoscope By S.H. McGurrin
; Syntax v5,n01 p.19
;================================
@Scope
	CALL Release		;Wait for no key press
	CALL Rnd8Bit
	AND 01			;Odd number?
	JR Z +2			;CLS with " "
	LD A,80			;CLS with "g "
	CALL CLS+3
@ScopeLup	
	LD A,3F			;Upper limit=63
	LD (Limit+1),A		;POKE upper limit
	CALL Modulus		;Get Xcoord, range=0 to 63
	LD C,A
	LD A,2F			;Upper limit=47
	LD (Limit+1),A		;POKE upper limit
	CALL Modulus		;Get Ycoord, range=0 to 47
	LD B,A			;BC=Y/Xcoords
	CALL Rnd8Bit		;Get a random number
        BIT 0,A            	;Plot or UnPlot pixels
        JR NZ +4
        LD A,40            	;40 FOR Plot
        JR +1
	XOR A              	;0 FOR UNPlot
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT X,Y
        LD A,2F			;47
        SUB B
        LD B,A
	POP AF 		   	;POP Plot/UnPlot Flag
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT X,47-Y
        LD A,3F			;63
        SUB C
        LD C,A
	POP AF		   	;POP Plot/UnPlot Flag
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT 63-X,47-Y
        LD A,2F			;47
        SUB B
        LD B,A
	POP AF		 	;POP Plot/UnPlot Flag
	CALL Plot		;PLOT 63-X,Y
	CALL KeySort
        JR ScopeLup
;================================
; Generate a random wallpaper pattern
;================================
@WallPaper
	CALL Release		;Wait for no key press
	LD HL,Limit+1	        
	LD (HL),0A		;Set upper random number limit to 10
	CALL Rnd8Bit		;Get 8bit random number
	AND 01			;A=0 or 1
	LD C,A
	JR Z +2			;A=0, move on
	LD C,80			;Yes, C=128 for graphic chrs
	LD HL,Pattern		;Pattern data address
	LD B,07			;Byte count
@NxtChr
	PUSH HL			;PUSH Pattern address
	CALL Modulus		;A=1 to 9
	POP HL			;POP Pattern address
	ADD A,C			;Add 0 or 128 to A
	LD (HL),A		;Store chr code
	INC HL			;Increase pattern addr
	DJNZ NxtChr		;Loop until B=0
@PrintWall
	XOR A			;Initialize counter
	LD DE,altDFile+1	;Destination=workarea
	LD HL,Pattern		;Source
	LD BC,0007		;Byte count
	LDIR			;Copy pattern to workarea
	INC A			;Next workarea address
	CP 71			;Repeat 113 times
	JR NZ PrintWall+4	;Loop until A=71
	LD B,18			;Install the N/L chrs to workarea
	LD HL,altDFile		;Workarea-1
	LD DE,0021		;Row count
	ADD HL,DE		;Next row
	LD (HL),76		;Set N/L
	DJNZ -5			;Loop until B=0
	CALL SwapDF
	CALL KeySort		;Check keys
	JR WallPaper+3		;Loop until "Q" pressed
;================================
; Set print chr to (BKG) or (Cell)
; based on random number generated.
;================================
@PrintCell
	CALL Rnd8Bit		;Get random number
	LD L,A			;L=8-Bit random number, range 0 to 255
	LD A,(Density)		;Get density value
	CP L			;IF L>Density THEN PRINT "gO" ELSE PRINT " "
	LD A,(Cell)		;Get chr to print
	JR C +3
	LD A,(BKG)		;ELSE PRINT " "
	LD (PokeA+1),A		;POKE chr to print
	RET
;================================
; Plot Routine
; On entry...
;  B=YCoord, 0 to 43
;  C=XCoord, 0 to 63
;  A=40=PLOT
;  A=00=UNPLOT
;===============================
@Plot
	PUSH BC			;PUSH X,Y coords
	PUSH AF			;PUSH Plot/UnPlot flag
	LD A,87			;A=87=NW Pixel
	BIT 0,B			;Y coord
	JR NZ +2		;Odd line number?
	ADD A,10		;A=97=SW Pixel 
	BIT 0,C			;X coord
	JR Z +2			;Even col number?
	ADD A,08		;A=8F=NE Pixel or A=9F=SE Pixel
	LD D,A			;D=Pixel quadrant value
	POP AF			;POP Plot/UnPlot flag
	ADD A,D			;A=D IF UNPlot or D+64 IF Plot
	LD (PlotPX+1),A		;Self modifying code
	SRL B			;Convert Y-coord to Print coord, Y/2
	SRL C			;Convert X-coord to Print coord, X/2
	LD HL,(DFile)		;Get DFile
	INC HL			;Start of DFile
	LD D,00
	LD E,C			;DE=Column number
	ADD HL,DE		;HL=Column position in DFile
	LD A,17			;23-Y/2
	SUB B
	LD B,A			;B=Row number
	JR Z +6
	LD DE,0021		;Bytes/row
	ADD HL,DE
	DJNZ -3
	LD A,(HL)		;Get chr at DFile addr
	CP 08			;LO Pixel Chr
	JR C PlotPX
	CPL			;HI Pixel Chr
@PlotPX				
	SET 00,A		;00=SR SET/RES X,A
	BIT 3,A
	JR Z +5
	CPL
	AND 8F
	JR +2
	AND 0F			;Low value PIXEL, codes 1 to 7
	LD (HL),A		;Hi value PIXEL, codes 128 to 135
	POP BC			;POP X,Y coords
	RET
;==============================
; Analyze current cell's 
; condition
;==============================
@Analyze
	PUSH BC 		;PUSH row counter
	PUSH HL 		;PUSH cell addr.
	LD DE,0021 		;Row length + N/L
	AND A 			;Clear CY
	SBC HL,DE 		;N cell
	LD A,76 		;N/L char.
	LD BC,0000 		;Row offset 0000
	INC (HL) 		;N cell value
	DEC HL 			;NW cell
	CP (HL) 		;N/L?
	JR NZ +4		;No, move on
	LD BC,0020 		;Yes
	ADD HL,BC 		;Wrap right
	INC (HL) 		;NW cell value
	ADD HL,DE 		;W cell
	INC (HL) 		;W cell value
	ADD HL,DE 		;SW cell
	INC (HL) 		;SW cell value
	AND A 			;Clear CY
	SBC HL,BC 		;BC=00 or 20 if wrap right 
	LD BC,0000 		;Row offset 0000
	INC HL 			;S cell
	INC (HL) 		;S cell value
	INC HL 			;SE cell
	CP (HL) 		;N/L?
	JR NZ +6		;No, move on
	LD BC,0020 		;Yes
	AND A 			;Clear CY
	SBC HL,BC 		;Wrap left
	AND A 			;Clear CY
	INC (HL) 		;SE cell value
	SBC HL,DE 		;E cell
	INC (HL) 		;E cell value
	SBC HL,DE 		;NE cell
	INC (HL) 		;NE cell value
	POP HL 			;POP cell addr.
	POP BC 			;POP row counter
	RET 
;==============================
;LIFE algorithm for globe
;==============================
@Global
	LD DE,altDFile
	LD HL,(DFile)		;HL=DFile
	LD BC,0319 		;Byte count
	LDIR 			;Transfer to altDFile
	LD HL,altDFile		;Use altDFile for analysis
	LD B,18 		;Row counter
@GlobLup
	INC HL 			;Next byte in row
	BIT 6,(HL) 		;N/L?
	JR NZ +7		;Yes, skip "O" test
	BIT 5,(HL) 		;"O"?
	CALL NZ Analyze		;Yes, analyze cell
	JR GlobLup		;No, loop
	DJNZ GlobLup		;Loop until B=0
	NOP			;Self modifying code, value=C9 or 00
@Update				;Update workarea rows 24 then 0
	LD DE,altDFile
	LD B,20 		;Byte counter, 32 chrs/row
	LD DE,Workarea		;DE=Workarea
	LD HL,0318 		;DFSize
	ADD HL,DE 		;
	INC HL 			;HL=row 24, update row 24 
	INC DE 			;DE=row 0
	LD A,(DE) 		;A=(row 0)
	ADD A,(HL) 		;A=(row 0)+(row 24) analysis
	LD (HL),A 		;Set row 24 analysis
	XOR A			;A=0
	LD (DE),A 		;Clear Row 0
	DJNZ -9			;Loop until B=0
	LD DE,altDFile		;Update row 0
	LD HL,0318 		;DFSize
	ADD HL,DE 		;
	EX DE,HL 		;DE=row0
	LD B,20 		;Byte counter, 32 chrs/row
	INC HL 			;HL=row 1
	INC DE 			;DE=row 25
	LD A,(DE) 		;A=(row 25)
	ADD A,(HL) 		;A=(row 25)+(row 1)
	LD (HL),A 		;Set row 1 analysis
	XOR A			;A=0
	LD (DE),A 		;Clear Row 25
	DJNZ -9			;Loop until B=0
;==============================
;Analysis completed
;==============================
	LD HL,altDFile
	INC HL 			;HL=row 1
	LD B,18			;Row count 
@CellCount
	LD A,(HL)		;Analysed cell value
	CP 03			;IF A=03, 36 or 37 THEN Birth
	JR Z Birth
	CP 36
	JR Z Birth
	CP 37
	JR Z Birth
	LD A,(BKG)		;Print (BKG)
	JR Death
@Birth
	LD A,(Cell)		;Print (Cell)
@Death
	LD (HL),A
	INC HL 			;Next altDFile
	BIT 6,(HL) 		;EOL?
	JR Z CellCount		;No, loop until EOL
	INC HL 			;Skip EOL marker
	DJNZ CellCount		;Loop until B=0
;================================
; This routine makes for smoother
; animation as it prevents the "ripple"
; effect seen when updating the display
;================================
@SwapDF
	LD HL,(DFile)		;Get DFile
	PUSH HL			;PUSH DFile
	EX DE,HL		;DE=DFile
	LD HL,altDFile		;HL=alt DFile
	LD (DFile),HL		;Set altDFile as (DFile)
	LD BC,0319		;Byte count
	LDIR			;Copy altDFile to normal DFile
	POP HL			;POP normal DFile
	LD (DFile),HL		;Reset DFile pointer
	RET
;================================
; Test for keypress.
; On exit...
;  A=key chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; Wait for key release
;================================
@Release
	CALL Keyboard		;Call ROM scan keyboard routine.
	INC L			;Check if user's finger on key.
	JR NZ Release		;If so, go back and check again.
@Ret0
	LD BC,0000
	RET			;RET to BASIC, BC=0000
;================================
; Sort out the key presses
;================================
@KeySort
	LD BC,0001	 	;Delay Loop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ -5
	CALL Keyscan
	CP 01
	RET Z
	LD BC,4D08		;Jumptable base addr, four fold symmetry, style 1
	CP "A"			;"A" key?
	JR NZ +2		;No, move on
	POP HL			;Clear stack
	RET
	CP "S"			;"S" key?
	JR NZ +4		;No, move on
	POP HL			;Clear stack
	LD C,0A
	RET
	CP "D"			;"D" key?
	JR NZ +4		;No, move on
	POP HL			;Clear stack
	LD C,0C			;Random screen
	RET
	CP "F"			;"F" key?
	JR NZ +4		;No, move on
	POP HL			;Clear stack
	LD C,0F			;Kaleidoscope
	RET
	CP "G"			;"G" key?
	JR NZ +4		;No, move on
	POP HL			;Clear stack
	LD C,12			;Wallpaper
	RET
	CP "Z"			;"Z" key?
	JR NZ +4		;No, move on
	POP HL			;Clear stack
	LD C,15			;LifeA
	RET
	CP "X"			;"X" key?
	JR NZ +4		;No, move on
	POP HL			;Clear stack
	LD C,17			;LifeB
	RET
	CP "C"			;"C" key?
	JR NZ +4		;No, move on
	POP HL			;Clear stack
	LD C,19			;LifeC
	RET
	CP "N"			;"N" key?
	JR NZ +5		;No, move on
	LD HL,KeySort+2		;Delay counter
	INC (HL)		;Increase delay count
	RET
	CP "M"			;"M" key?
	JR NZ +8		;No, move on
	LD HL,KeySort+2		;Delay counter
	XOR A			;Test for 0 condition.
	CP (HL)	
	RET Z			
	DEC (HL)		;Decrease delay count
	RET
	CP "P"			;"P" key?
	JR NZ +17
@ISet
	LD A,FE			;Initialize A to FE
	INC A			;Set for next Ireg value
	INC A
	CP 20			;Out of ROM space?
	JR NZ +1		;No, move on
	XOR A			;Yes, cycle back to 0
	LD (ISet+1),A		;Store Ireg value
	LD I,A			;Set Ireg
	JP Release		;Return via wait for key release
	CP "O"			;"O" key?
	JR NZ +19
	LD A,I			;Get current Ireg value
	CP 1E			;ROM Ireg?
	JR Z +4			;No, move on
	LD A,1E			;Restore Ireg to ROM value
	JR -19			;Set Ireg
	LD A,(ISet+1)		;Get last used Ireg value
	CP FE			;FE? Ireg not changed yet
	JR Z ISet+2		;Yes, set Ireg to 0
	JR -28			;No, set Ireg to last value
	CP "W"			;"W" key?
	JR NZ +3		;No, move on
	JP Release		;Return via wait for key release
	CP 76			;"NL" key?
	JR Z +4
	CP "Q"
	RET NZ
	LD A,1E			;Reset Ireg just in case
	LD I,A
	POP HL			;Clear the stack
	LD BC,0000
	RET
;================================
; 8-Bit Random Number Generator
; Reg BC not used
; On exit...
;  Reg A=random number in random
;        0 to 255
;================================
@Rnd8Bit
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+1),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	RET
;================================
; Modulus code to ensure 
; number in desired range.
; On exit...
;  A=random number between 0 and
;    upper limit
;================================
@Modulus
	CALL Rnd8Bit		;Get 8-Bit random number
	LD H,00			
	LD L,A			;HL=8-Bit random number
@Limit
	LD A,00			;POKEd upper limit value
	LD D,H
	LD E,A			;Set DE to dividend
@RandLoop
	OR A			;Clear CY
	SBC HL,DE		;Subtract dividend from HL
	JP P RandLoop		;Keep doing it till negative
	ADD HL,DE		;Add the divided back again
	LD A,L			;Return results in A
	RET
;================================
; Locate print position in DFile
; then print POKEd chr.
; On entry...
;  B=Ycoord
;  C=Xcoord
; On exit
;  HL=DFile address
;  DE unchanged
;  BC unchanged
;  AF unchanged
;================================
@PrintX
	PUSH DE
	PUSH BC
	INC B			;Ycoord must be > 0
	LD HL,(DFile)		;Get start of DFile
	INC HL
	LD DE,0000		;Initialize DE
@RowCalc
	ADD HL,DE		
	LD E,21			;DE=33 bytes/row
	DJNZ RowCalc		;Add Bx33 to start of DFile
@AddCol
	LD E,C			;DE=Xcoord offset
	ADD HL,DE		;Add offset to DE
@PokeA
	LD (HL),00		;POKEd chr to print
	POP BC			;POP Y/Xcoords
	POP DE
	RET
;================================
; Print A to the four quadrants
; of the screen.
; On entry...
;   A=chr to print
;  BC=Y/Xcoords of NW quad
;================================
@PrintQuads
	CALL PrintX		;Print to NW quadrant B,C
	LD A,1F
        SUB C
        LD C,A
	CALL PrintX		;Print to NE quadrant B,31-C
	LD A,17
        SUB B
        LD B,A
	CALL PrintX		;Print to SE quadrant 25-B,31-C
	LD A,1F			
        SUB C
	LD C,A
	CALL PrintX		;Print to SW quadrant 25-B,C
	RET
;================================
; Clear screen with background
; character.
;================================
@CLS
	LD A,(BKG)
	LD D,A
	LD HL,(DFile)
	LD BC,0318		;Byte count
@CLSLup
	INC HL
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip it
	LD (HL),D		;Clear this byte with BKG chr.
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ CLSLup		;Loop until BC=0
	RET