;=================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;=================================

ORG 4084

@LADDR=09D8
@DFile=400C
@Seed=4032

@Size=8000				;32768
@Table=8002				;32770
@Count=8004				;32772
@Delay=8005				;32773
@Frame01=800A				;32778

JP Clear				;16516
JR Encode				;16519
JR Store				;16521
JP Lookup				;16523

;=================================
;Encoder for screen compression by XAVSNAP
;Revised 09/15 by GCH
;=================================
@Encode
	LD DE,Buffer+9
	LD HL,Buffer+8			;Set source
	LD (HL),00			;Fill buffer with " "
	LD BC,0300			;Byte Count, 768
	LDIR				;Clean data buffer.
	CALL Main
	EX DE,HL			;HL=Buffer address
	LD (HL),40			;Exit code
	INC HL
	LD (HL),01
	LD DE,Buffer+8
	SBC HL,DE
	INC HL
	LD (FrameSize+1),HL		;Set frame size
	LD B,H				;Set BC to frame size
	LD C,L				
	RET				;Return size to BASIC
;==================================
@Main
	LD DE,Buffer+8			;Buffer offset
	LD HL,(DFile)			;Get DFile
	INC HL
	LD B,18				;Row counter, 24
@NextChr
	LD A,(HL)			;Chr to evaluate
	LD C,00				;Initialize chr counter
;==================================
;Return if the Counter reaches 255
;N/L ignored.
;==================================
@ChrLoop
	INC C				;Increase chr counter
	INC HL				;DFile address
	EX AF				;Save chr code
	LD A,(HL)			;Get (HL)
	CP 76				;N/L?
	JR NZ NxtLine			;No, move on
	DJNZ NotDone			;Process 24 rows
;Write last 1 or 2 data bytes to Buffer
	LD A,01				;Single chr
	CP C				;Character Count
	DEC HL				;Last DFile address
	LD A,(HL)			;Get last chr code
	JR Z Last1			;Single chr?
	OR 40				;No, multi chr
	LD (DE),A			;Save multi chr code
	INC DE
	LD A,C
@Last1
	LD (DE),A			;Save single char. or multi. char. Count
	INC DE				;Next buffer addr
	RET
@NotDone
	INC HL				;Video address
@NxtLine
	LD A,C				;A=Counter
	CP FF				;Max chr count reached?
	JR Z EndFF			;Yes, write FE then reset counter
	EX AF				;Restore chr code
	CP (HL)				;Compare (HL)
	JR Z ChrLoop			;Loop if the same chr code
@EndLoop
	EX AF				;Save chr counter
@EndFF
	DEC HL				;Backup DFile
	OR A				;Clear CY?
	LD A,C				;A=chr count
	SUB 2				;A>1?
	JR NC TwoBytes			;Yes, more than 2 bytes
	EX AF				;No, restore chr counter
@OneByte
	LD (DE),A			;Write single char.
	INC DE				;Next buffer addr
	INC HL				;Next DFile addr
	JR NextChr
@TwoBytes
	EX AF				;AF=Value, write 2 bytes
	OR 40				;Multi tag+chr code, 40+code
	LD (DE),A			;Set multi value
	INC DE				;Next buffer addr.
	LD A,C				;Chr count
	JR OneByte			;Set byte count
;==============================
;Store compressed data to (Seed)
;value
;==============================
@Store
	LD HL,Buffer+8			;Source=REM 5 "data address"
	LD DE,(Seed)			;Get destination from (Seed), upper RAM address
@FrameSize
	LD BC,0000			;Bytes to move, POKE'd from Encode routine.
	PUSH BC				;PUSH size
	LDIR				;Copy frame
	LD (Table),DE			;Set lookup table address
	POP BC				;POP size
	LD HL,(Size)			;Get byte count
	ADD HL,BC			;Add size to byte count
	LD (Size),HL			;Set byte count
	LD BC,0000			;Set for return
	RET
;=================================
;Prep upper RAM for data
;=================================
@Clear
	LD HL,8000			;Source
	LD DE,8001			;Destination
	LD BC,3FF0			;Byte count
	LD (HL),00			;Set (HL) to 0
	LDIR				;Fill with 0
	RET
;=================================
;Create lookup table.
;==================================
@Lookup
	LD DE,Frame01			;Address of 1st frame
	LD HL,(Table)			;Get start of lookup table
	LD (HL),E			;Set frame1 addr LO
	INC HL				;Next lookup table addr
	LD (HL),D			;Set frame1 addr HI
	EX DE,HL			;Swap frames & lookup
	LD BC,(Size)			;Get byte count
@NxtFrame
	INC DE				;Next position in lookup table
@Not4001
	LD A,40				;End of frame marker
	CPIR				;Search for 01
	JP PO LookEnd			;JP when BC=0
	LD A,01				;Test for the 40,01 end of frame combo?
	CP (HL)				;01?
	JR NZ Not4001			;No, keep looking else store the address in table
	INC HL
	EX DE,HL			;Swap frames & lookup table
	LD (HL),E			;Set frameX addr LO
	INC HL				;Next lookup table addr
	LD (HL),D			;Set frameX addr HI
	EX DE,HL			;Swap frames & lookup table
	DEC HL
	JR NxtFrame			;Loop until BC=0
@LookEnd
	LD B,D				;BC=End of tabble addr
	LD C,E
	RET				;Return end of table addr to BASIC
@Buffer
